/*
 *  Copyright (C) 2003  Jens Kanschik,
 * 	mail : jensKanschik@users.sourceforge.net
 *
 *  Part of <hypergraph>, an open source project at sourceforge.net
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package hypergraph.applications.google;

import hypergraph.graphApi.Element;
import hypergraph.graphApi.Graph;
import hypergraph.graphApi.GraphException;
import hypergraph.graphApi.Group;
import hypergraph.graphApi.Node;

import java.applet.AppletContext;
import java.awt.Color;
import java.awt.event.MouseEvent;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.JApplet;

import com.google.soap.search.GoogleSearch;
import com.google.soap.search.GoogleSearchFault;
import com.google.soap.search.GoogleSearchResult;
import com.google.soap.search.GoogleSearchResultElement;

/**
 * @author Jens Kanschik
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class GraphPanel extends hypergraph.visualnet.GraphPanel {

	private static Group	groupNew;
	private static Group	groupPending;
	private static Group	groupDone;
	private static String	homepageUrl = "http://hypergraph.sourceforge.net";
	private JApplet			explorer;
	
	
	public GraphPanel(Graph graph,JApplet explorer) {
		super(graph);
		this.explorer = explorer;
		groupNew = graph.createGroup();
		groupPending = graph.createGroup();
		groupDone = graph.createGroup();
		graph.getAttributeManager().setAttribute(NODE_FOREGROUND, groupNew, Color.lightGray);
		graph.getAttributeManager().setAttribute(NODE_FOREGROUND, groupPending, Color.green);
		graph.getAttributeManager().setAttribute(NODE_FOREGROUND, groupDone, Color.black);
	}


	static String clientKey = "ANukEd5QFHLuNxEECaFpJlaUikqQ8fn9";

	static String url;	
	public void addRelatedUrls(String startUrl,final int level) {
		url = startUrl;
		Runnable r = new Runnable() {
			public void run() {
				loadRelatedUrls(url,level);
			}
			};
		Thread t = new Thread(r);
		t.start();
	}

	public void loadRelatedUrls(String startUrl,int level) {
		if (level < 0)
			return;
		// mark root node that the search has started
		// if the node is not new (i.e. either done or pending),
		// we don't load the URLs because it has been done or is currently in progress.
		Node root = (Node) getGraph().getElement(startUrl);
		if (root != null) {
			if (!root.getGroup().equals(groupNew))
				return;
			root.setGroup(groupPending);
		}
		
		// Create a Google Search object, set our authorization key
		GoogleSearchResult searchResult = null;
		GoogleSearch s = new GoogleSearch();
		s.setKey(clientKey);
		try {
			s.setQueryString("related:" + startUrl);
			searchResult = s.doSearch();
//			System.out.println("Google Search Results:");
//			System.out.println("======================");
//			System.out.println(searchResult.toString());
		} catch (GoogleSearchFault f) {
			System.out.println(	"The call to the Google Web APIs failed: \n" +
				f.toString());
			return;
		}
		GoogleSearchResultElement[] elements = searchResult.getResultElements();
		for (int i = 0; i < elements.length; i++) {
			Node n = (Node) getGraph().getElement(elements[i].getURL());
			if (n == null) {
				try {
					synchronized (getGraph()) {
						n = getGraph().createNode(elements[i].getURL());
						if (!n.equals(root))
							n.setGroup(groupNew);
					}
				} catch (GraphException ge) {
					ge.printStackTrace();
				}
				n.setLabel(elements[i].getTitle());
			}
			if (i == 0)
				root = n;
			else
				synchronized (getGraph()) {
					getGraph().createEdge(root, n);
				}
			try {Thread.sleep(5);}catch(Exception e) {}
			addRelatedUrls(n.getName(), level - 1);
		}
		if (root != null)
			root.setGroup(groupDone);
	}


	public void setHoverElement(Element element, boolean repaint) {
		if (getHoverElement() != element) {
			if (element == null) {
				AppletContext context = explorer.getAppletContext();
				context.showStatus("");
				setToolTipText(null);
			}
			// show a tooltip (xlink:href) if the element is a node
			if (element != null && element.getElementType() == Element.NODE_ELEMENT) {
				AppletContext context = explorer.getAppletContext();
				String href = element.getName();
				context.showStatus(href);
				setToolTipText(href);
			}
			// do something different if the element is an edge
			if (element != null && element.getElementType() == Element.EDGE_ELEMENT) {
			}
		}
		super.setHoverElement(element, repaint);
		if (getHoverElement() == null)
			explorer.getAppletContext().showStatus("");
	}
	protected void logoClicked(MouseEvent e) {
		super.logoClicked(e);
	    try {
			explorer.getAppletContext().showDocument(
	    		new URL(homepageUrl));
	    } catch (MalformedURLException mue) {}
	}
	
	public void mouseMoved(MouseEvent e) {
		super.mouseMoved(e);
		if (isOnLogo(e.getPoint())) 
			explorer.getAppletContext().showStatus(homepageUrl);
	}
	public void nodeClicked(int clickCount,Node node) {
		super.nodeClicked(clickCount,node);
		if (clickCount == 1)
			addRelatedUrls(node.getName(), 0);
	}

}
